/* Programme qui associe P4 et P5 pour piloter effectivement les moteurs avec le moniteur USB.
   L'affichage précise la valeur de consigne moteur après la valeur angulaire de position.

   • 'Q' Force la configuration 50 initialisée préalablement à l'encombrement réduit A et coupe les moteurs.
     (Provisoire, car dans la version définitive c'est la posture de rétractation qui sera initialisée.)
   • Pour récupérer la commande 'a', éteindre ou allumer l'écran se fait avec 'v' qui fonctionne en
     bascule de type OUI/NON. Les affichages sur le moniteur sont épurés.
   • La commande 'a' précise l'articulation sur laquelle on veut agir par incréments.
     0 : Sortir du mode, 1 : Serrage, 2 : Coude, 3 : Epaule, 4 : Hanche.
     Hors de cette plage l'Erreur n°7 sera générée.
   • La commande 'i' précise les incréments désirés sur la Consigne. (Exprimés en degrés)
     Les valeurs pour l'incrément devront être comprises entre [1 et 50].  (Ou erreur n°8.)
   • '-' Décrémente, '+' Incrémente la position angulaire du moteur.
     En mode pilotage d'un moteur par incréments l'écran affiché est spécifique.
     Toute commande autre que 'a', 'i', '+' et '-' fait sortir du mode. (Ainsi que "a0".)
     Tout débordement des amplitudes de balatage moteur génère l'Erreur n°3.
     Attention : La commande 'a1' force l'ouverture de la pince.
     Dans cette version, la commande 's' ferme la pince en plus d'ajuster la valeur de serrage.
   • 'n' neutralise ou réactive les moteurs.(bascule OUI/NON) Une LED double précise l'état :
         Rouge branchée sur D3 prévient que les moteurs sont actifs. (Rouge car danger mécanique.)
         Verte branchée sur D4 signale que les moteurs sont Neutralisés. 

   UTILISER LE PROGRAMME : Frapper une chaine de caractères sur la ligne USB au format présenté dans
       P4, mais cette fois les moteurs réagissent si ils sont correctement branchés sur le multiplexeur.
       Pour l'erreur n°3 ce n'est plus l'analyseur syntaxique qui effectue la vérification, mais la procédure
       Positionne_un_moteur(), car chaque effecteur présente un débattement différent.
   
   Erreur 1 : Chaîne inférieure à un caractère. (Donc uniquement la validation.)
   Erreur 2 : Caractère de commande invalide.
   Erreur 3 : Consigne de position angulaire hors de la fourchette du moteur concerné.
   Erreur 4 : Num enregistrement hors de la fourchette [1 à 50].
   Erreur 5 : Tenter de restituer un enregistrement vide.
   Erreur 6 : Commande 'Z' non suivie de la valeur 50.
   Erreur 7 : Commande 'a' hors limite [0 à 4].
   Erreur 8 : Commande 'i' hors limite [1 à 50].
   
   Pour des raisons pratiques les entrées et les sorties branchées sur le multiplexeur
   utilisent des ports bien séparés pour faciliter les branchements.
   (Longueur des fils, encombrement etc.) En fin de développement le choix est le suivant :
   Moteur de la HANCHE sur le statif sur S15,
   Moteur de l'EPAULE sur S0,
   Moteur du COUDE sur S4,
   Moteur de la PINCE sur S8.
   
   Le BUZZER actif est branché sur D2.
   La LED double couleur Rouge "Moteurs actifs" est branché sur D7.
   La LED double couleur Verte "Moteurs neutralisés" est branché sur D9.
   SDA est branché sur A4.  SCL est branché sur A5
                                                                PGM               RAM dynamique
 PGM de dialogue sur la ligne série.                    16706 Octets (54%)      974 Octets (47%)

----------------------- Déclarations pour le matériel --------------------------    */
#include <Wire.h> // Gère les protocole I2C sur A4 et A5.
#include <EEPROM.h>
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)
#include <Adafruit_PWMServoDriver.h>
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

//----------------------- Définition des broches d'E/S -------------------------
#define BUZZER 2 // Le bruiteur est piloté par la sortie binaire D2.
#define LED_Moteur_ON 7 // LED (Double) Rouge qui signale que les moteurs sont actifs.
#define LED_Moteur_OFF 9 // LED (Double) Verte qui signale que les moteurs sont neutralisés.

//-------------------------- Constantes du programme ---------------------------
#define Version_PGM "13-10-2019"    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define LGR_chaine 7 // Longueur maximale de la chaine de dialogue.
const byte TailleTexte = LGR_chaine + 1; // Taille réservée pour LGR_MAX plus le délimiteur.

//--------------------------- Variables du programme ---------------------------
boolean Ecran_en_veille;
boolean Mode_incremental;
boolean Moteurs_ON;
boolean Ouverte;
boolean Sens_plus;

char Angle_Serrage_Pince;
char Caractere;
char Coude;
char Epaule;
char Hanche;
int DataX;

byte Compteur_caracteres;
byte Position_curseur;
byte Increment;
byte Moteur_incremente;
byte Nb_Enregistrements;
byte num_ERR;

int Consigne;
int Consigne_Coude;
int Consigne_Epaule;
int Consigne_Hanche;
int Consigne_Pince;
int Valeur_Numerique; // Contiendra la transformée numérique d'une chaine.
int PTR;

char Chaine_Memorisee[TailleTexte]; // Tampon mémoire pour la chaine entrante.
//------------------------------------------------------------------------------

void setup() {
  u8g.setFont(u8g_font_6x10); // Pocice de matrices de 6x10 pixels lettres MJA/min comprises.
  pwm.begin(); // Initialise le module multiplexeur.
  pwm.setPWMFreq(50); // Fréquence de la PWM : En standard 50Hz pour des servomoteurs.  
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW);
  pinMode(LED_Moteur_ON, OUTPUT); digitalWrite(LED_Moteur_ON, LOW); Moteurs_ON = false;
  pinMode(LED_Moteur_OFF, OUTPUT); digitalWrite(LED_Moteur_OFF, HIGH);
  Serial.begin(115200); Serial.print("Version du "); Serial.println(Version_PGM);
  Moteur_incremente = 1; Increment = 1;
  Ecran_en_veille = false; u8g.sleepOff(); // Active l'écran;
  Angle_Serrage_Pince = 60; Ouverte = false; Consigne_Pince = 310; DataX = 73;
  Coude = 0; Consigne_Coude = 270; Epaule = 0; Consigne_Epaule = 406; Hanche = 0; Consigne_Hanche = 302;
  Calcule_Nb_Enregistrements(); Mode_incremental = false; PAGE1();}

void loop() {
  Attendre_une_chaine();
  Serial.print('['); Serial.print(Caractere); Serial.print(Chaine_Memorisee); Serial.print("] > ");
  Analyseur_syntaxique();
  if (Mode_incremental) PAGE2(); else PAGE1();} // Traite la consigne. 

void Neutralise_les_moteurs() {
  Moteurs_ON = ! Moteurs_ON;
  if (Moteurs_ON) {digitalWrite(LED_Moteur_ON, HIGH); digitalWrite(LED_Moteur_OFF, LOW);
                   pwm.setPWM(13,0, Consigne_Coude); pwm.setPWM(0,0, Consigne_Epaule);
                   pwm.setPWM(1,0, Consigne_Hanche); pwm.setPWM(0,0, Consigne_Epaule);
                   pwm.setPWM(15,0, Consigne_Pince);}
  else {digitalWrite(LED_Moteur_ON, LOW); digitalWrite(LED_Moteur_OFF, HIGH);}}

void QUITTER() {
  // Place le bras manipulateur en configuration "Rétractée A".
  // Ouverture de la pince ne pas contracter en permanence le ressorts.
  Ecran_en_veille = false; Moteurs_ON = true; Caractere = 'r'; Valeur_Numerique = 50; Traite_Consigne();
  Neutralise_les_moteurs(); Caractere = 'v'; num_ERR = 0; Traite_Consigne();} // Place l'écran en veille.
  
void Calcule_la_consigne_moteur(byte Articulation) {// 1 Serrage, 2 Coude, 3 Epaule, 4 Hanche.
  int  Position_min, Position_MAX, Consigne_min, Consigne_MAX;
  switch (Articulation) {
     case 1 : {Position_min = 30; Position_MAX = 90; Consigne_min = 373; Consigne_MAX = 264; break;}
     case 2 : {Position_min = -40; Position_MAX = +90; Consigne_min = 200; Consigne_MAX = 440; break;}
     case 3 : {Position_min = 0; Position_MAX = +90; Consigne_min = 406; Consigne_MAX = 218; break;}
     case 4 : {Position_min = -90; Position_MAX = +90; Consigne_min = 463; Consigne_MAX = 144;}}
  if ((Valeur_Numerique > Position_MAX) || (Valeur_Numerique < Position_min)) num_ERR = 3;
  if(num_ERR == 0) Position_curseur = map(Valeur_Numerique, Position_min, Position_MAX, 9, 118);
  Consigne = map(Valeur_Numerique, Position_min, Position_MAX, Consigne_min, Consigne_MAX);
  Serial.println(); Serial.print("Consigne = "); Serial.print(Consigne);}
 
void Positionne_un_moteur(byte Articulation) {         
  // 1 Serrage, 2 Coude, 3 Epaule, 4 Hanche.
  byte Sortie;
  if (Caractere == 's') {Ouverte = false; Angle_Serrage_Pince = Valeur_Numerique;}
  if (Caractere == 'o') Valeur_Numerique = 90;
  if (Caractere == 'f') Valeur_Numerique = Angle_Serrage_Pince;
  Calcule_la_consigne_moteur(Articulation);
  if (num_ERR == 0) {
     switch (Articulation) {
        case 1 : {if (Ouverte) Consigne_Pince = 264; else Consigne_Pince = Consigne; Sortie = 8; break;}
        case 2 : {Coude = Valeur_Numerique; Consigne_Coude = Consigne; Sortie = 4; break;}
        case 3 : {Epaule = Valeur_Numerique; Consigne_Epaule = Consigne; Sortie = 0; break;}
        case 4 : Hanche = Valeur_Numerique; Consigne_Hanche = Consigne; Sortie = 15;}
  if (Moteurs_ON) pwm.setPWM(Sortie,0, Consigne);}}

void Traite_Restitution() {
  Verifie_1_a_50();
  if (num_ERR == 0) {
     Positionne_PTR_en_fonction_de_Num(); // Pointe le début de l'enregistrement.
     if (!Lire_un_Octet_en_EEPROM()) num_ERR = 5; // Vérifie que l'enregistrement n'est pas vide.
     if (num_ERR == 0) {
         Ouverte = Lire_un_Octet_en_EEPROM(); Angle_Serrage_Pince = Lire_un_Octet_en_EEPROM();
         if (Ouverte) Caractere = 'o'; else Caractere = 'f'; Traite_Consigne();
         DataX = Lire_un_Entier_en_EEPROM();
         Valeur_Numerique = Lire_un_Entier_en_EEPROM(); Caractere = 'c'; Traite_Consigne();
         Valeur_Numerique = Lire_un_Entier_en_EEPROM(); Caractere = 'e'; Traite_Consigne();
         Valeur_Numerique = Lire_un_Entier_en_EEPROM(); Caractere = 'h'; Traite_Consigne();
         Serial.println();}}} 

void Gere_le_mode_incremental() {
  // 0 : Sortir du mode, 1 : Serrage, 2 : Coude, 3 : Epaule, 4 : Hanche.
  if ((Valeur_Numerique < 0) || (Valeur_Numerique > 4)) num_ERR = 7;
  if(num_ERR == 0) {
    Mode_incremental = (Valeur_Numerique != 0);
    if (Mode_incremental) {Moteur_incremente = Valeur_Numerique;
       switch (Moteur_incremente) {
         case 1 : {Valeur_Numerique = 90; Angle_Serrage_Pince = 90; Position_curseur = 118; Ouverte = true; break;}
         case 2 : {Valeur_Numerique = Coude; break;}
         case 3 : {Valeur_Numerique = Epaule; break;}
         case 4 :  Valeur_Numerique = Hanche;}
       Calcule_la_consigne_moteur(Moteur_incremente);}}}

void Gere_la_valeur_incrementale() {
  if ((Valeur_Numerique < 1) || (Valeur_Numerique > 50)) num_ERR = 16;
  if(num_ERR == 0) Increment = Valeur_Numerique; Mode_incremental = true;}
  
void Effectue_un_pas() {
  Mode_incremental = true;
    switch (Moteur_incremente) {
    case 1 : {Valeur_Numerique = Angle_Serrage_Pince; Ouverte = false; break;}
    case 2 : {Valeur_Numerique = Coude; break;}
    case 3 : {Valeur_Numerique = Epaule; break;}
    case 4 : Valeur_Numerique = Hanche;}
  if (Sens_plus) Valeur_Numerique = Valeur_Numerique + Increment;
            else Valeur_Numerique = Valeur_Numerique - Increment;
  Positionne_un_moteur(Moteur_incremente);
  if ((Moteur_incremente == 1) && (num_ERR == 0)) Angle_Serrage_Pince = Valeur_Numerique;}
  
void Traite_Consigne() {// Les commandes sont classées par ordre alphabétique.
  Mode_incremental = false;
  switch (Caractere) {
    case '+' : {Sens_plus = true; Effectue_un_pas(); break;}
    case '-' : {Sens_plus = false; Effectue_un_pas(); break;}
    case 'a' : {Gere_le_mode_incremental(); break;} // Passe en mode INCREMENTAL.
    case 'c' : {Positionne_un_moteur(2); break;} // Positionne le COUDE.
    case 'e' : {Positionne_un_moteur(3); break;} // Positionne le EPAULE.
    case 'f' : {Ouverte = false; Positionne_un_moteur(1); break;} // Ferme la pince.
    case 'h' : {Positionne_un_moteur(4); break;} // Positionne la HANCHE.
    case 'i' : {Gere_la_valeur_incrementale(); break;} // Impose la valeur de l'incrément.
    case 'k' : {Kill_enregistrement(); break;} //Effacer un enregistrement.
    case 'L' : {Lister_EEPROM(); break;}
    case 'm' : {Memorise_Posture(); break;}
    case 'n' : {Neutralise_les_moteurs(); break;} 
    case 'o' : {Ouverte = true; Positionne_un_moteur(1); break;} // Ouvre la pince.
    case 'x' : {DataX = Valeur_Numerique; break;} // Mémorise la donnée F.E.
    case 'Q' : {QUITTER(); break;} // Laisser l'ensemble en configuration "propre".
    case 'r' : {Traite_Restitution(); break;}
    case 's' : {Positionne_un_moteur(1); break;} // Ferme la pince à la valeur de serrage infiquée.
    case 'u' : {Liste_les_enregistrements_utilises(); break;}
    case 'v' : {Ecran_en_veille = ! Ecran_en_veille;  // Inverser l'état de "Ecran en veille".
                if (Ecran_en_veille) u8g.sleepOn(); else u8g.sleepOff(); break;}
    case 'Z' : {Efface_toutes_les_postures_en_EEPROM(); break;}
    default : num_ERR = 2;}}

void BIP() {digitalWrite(BUZZER, HIGH); delay(100); digitalWrite(BUZZER, LOW);}

void Analyseur_syntaxique() {// Compteur_caracteres mis à jour par Attendre_une_chaine()
  num_ERR = 0;
  Serial.print(" Nb = "); Serial.print(Compteur_caracteres);
  if (Compteur_caracteres == 0) num_ERR = 1;
  else {Compteur_caracteres--;
        //==================== Analyse de la commande =====================
  Traite_Consigne();} // Impérativement en premier car peut générer l'erreur 3 ou l'erreur 4 ...
  Serial.print(" C = "); Serial.print(Caractere); Serial.print(" Valeur = ");
  Serial.print(Valeur_Numerique); if (num_ERR == 0) Serial.println(" !OK!");
  else {BIP(); Serial.print(" !ERR "); Serial.print(num_ERR); Serial.println("!");}}

void Attendre_une_chaine() {
  boolean FINI; char Caractere_lu;
  FINI = false; Compteur_caracteres=0; // Si FIFO non vide lire les caractères :
  while(!FINI) {
    if (Serial.available()) {Caractere_lu = Serial.read();
        if (Caractere_lu == char(13)) FINI = true; // char (13) est le LF de "Valider". (Sentinelle.)
        if ((Compteur_caracteres < (LGR_chaine)) && (!FINI))
           {Chaine_Memorisee[Compteur_caracteres] = Caractere_lu; Compteur_caracteres++;
            delay(5);}}} // Délai impératif.
  Chaine_Memorisee[Compteur_caracteres] = '\0'; // Placer le délimiteur de fin de texte.
  Caractere = char(Chaine_Memorisee[0]);
  // Extraire le nombre. En sortie Valeur_Numerique contiendra le nombre dans la chaine
  // ou 00 si le texte n'est pas "numérique" : 
  for(byte J = 0; J < Compteur_caracteres; J++)
      Chaine_Memorisee[J] = Chaine_Memorisee[J+1]; // Enlève le caractère de contrôle. 
  Valeur_Numerique = atoi(Chaine_Memorisee);}
  
void PAGE1() {
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  CADRES(); u8g.drawVLine(85,0, 12 ); Aff_TEXTE(8, 10 ,"Etat du BRAS  Nb"); u8g.setPrintPos(109, 10);
  u8g.print(Nb_Enregistrements); Aff_TEXTE(5,21," PINCE :");
  if (Ouverte) Aff_TEXTE(60 ,21,"Ouverte");
  else {Aff_TEXTE(60,21,"Fermee a");
        u8g.setPrintPos(113, 21); u8g.print(int(Angle_Serrage_Pince));}
  Aff_TEXTE(11,31," Data X :"); u8g.setPrintPos(71, 31); u8g.print(DataX); 
  Aff_TEXTE(11,41,"  COUDE :"); u8g.setPrintPos(71, 41); Affiche_valeur_angulaire(Coude);
  u8g.print(Consigne_Coude); u8g.print(')');
  Aff_TEXTE(11,51," EPAULE :"); u8g.setPrintPos(71, 51); Affiche_valeur_angulaire(Epaule);
  u8g.print(Consigne_Epaule); u8g.print(')');
  Aff_TEXTE(11,61," HANCHE :"); u8g.setPrintPos(71, 61); Affiche_valeur_angulaire(Hanche);
  u8g.print(Consigne_Hanche); u8g.print(')');}
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.

void PAGE2() {
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  CADRES(); Aff_TEXTE(21, 10 ,"INCREMENTS de");  u8g.setPrintPos(103,10); u8g.print(Increment);
  Aff_TEXTE(21, 22 ,"Moteur :"); u8g.setPrintPos(76, 22);
  switch (Moteur_incremente) {
    case 1 : {u8g.print("Serrage"); break;}
    case 2 : {u8g.print("Coude"); break;}
    case 3 : {u8g.print("Epaule"); break;}
    case 4 : u8g.print("Hanche");}
  Aff_TEXTE(21, 35 ,"Consigne = "); u8g.setPrintPos(88,35);
  switch (Moteur_incremente) {
    case 1 : {u8g.print(Consigne_Pince); break;}
    case 2 : {u8g.print(Consigne_Coude); break;}
    case 3 : {u8g.print(Consigne_Epaule); break;}
    case 4 : u8g.print(Consigne_Hanche);}
  Aff_TEXTE(21, 48 ,"Position = "); u8g.setPrintPos(88,48); 
  switch (Moteur_incremente) {
    case 1 : {u8g.print(int(Angle_Serrage_Pince)); break;}
    case 2 : {Affiche_valeur_angulaire(Coude); break;}
    case 3 : {Affiche_valeur_angulaire(Epaule); break;}
    case 4 : Affiche_valeur_angulaire(Hanche);}
  u8g.drawHLine(1,25,126); u8g.drawHLine(1,38,126); u8g.drawHLine(1,51,126);
  // Tracer les rubans de représentation de la plage possible :
  u8g.drawHLine(9,54,110); u8g.drawHLine(9,55,110); u8g.drawHLine(9,56,110);
  u8g.drawPixel(Position_curseur,57); u8g.drawPixel(Position_curseur,58);
  u8g.drawPixel(Position_curseur,59); u8g.drawPixel(Position_curseur,60);}
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void Affiche_valeur_angulaire(int Valeur) {
  if (Valeur == 0)  u8g.print(' '); if(abs(Valeur) < 10) u8g.print(' '); if (Valeur > 0)  u8g.print('+'); 
  u8g.print(Valeur); if (!Mode_incremental) u8g.print(" (");}
  
//--------------------------- Routines de gestion EEPROMe ---------------------------
byte Lire_un_Octet_en_EEPROM() {PTR++; return eeprom_read_byte((unsigned char*) PTR - 1);}
     
int Lire_un_Entier_en_EEPROM() {PTR++; PTR++; return eeprom_read_word((unsigned int*) (PTR - 2));}

void Ecrire_un_Octet_en_EEPROM(byte Octet) {eeprom_write_byte((unsigned char*) PTR, Octet); PTR++;}

void Ecrire_un_Entier_en_EEPROM(int Entier)
     {eeprom_write_word((unsigned int*) PTR, Entier); PTR++; PTR++;}

void Verifie_1_a_50() {if ((Valeur_Numerique < 1) || (Valeur_Numerique > 50)) num_ERR = 4;}

void Positionne_PTR_en_fonction_de_Num() {PTR = 463; PTR = PTR + (11 * Valeur_Numerique);}

void Calcule_Nb_Enregistrements() {
  Nb_Enregistrements = 0;
  PTR = 474;
  while (PTR < 1023) {if (Lire_un_Octet_en_EEPROM()) Nb_Enregistrements++; PTR = PTR + 10;}}
  
void Efface_toutes_les_postures_en_EEPROM() {
  if(Valeur_Numerique != 50) num_ERR = 6;
  if (num_ERR == 0) {
      for (int I=459; I < 1024; I++) Ecrire_Octet_EEPROM(I, 00); Calcule_Nb_Enregistrements();}}
      
void Memorise_Posture() {
  Verifie_1_a_50();  // Num va de 1 à 50.
  if (num_ERR == 0) {
      // Les enregistrements commencent en adresse relative 474.
      Positionne_PTR_en_fonction_de_Num(); // Pointe le début de l'enregistrement.
      Ecrire_un_Octet_en_EEPROM(true); // Valide l'enregistrement.
      Ecrire_un_Octet_en_EEPROM(Ouverte); Ecrire_un_Octet_en_EEPROM(Angle_Serrage_Pince);
      Ecrire_un_Entier_en_EEPROM(DataX); Ecrire_un_Entier_en_EEPROM(Coude);
      Ecrire_un_Entier_en_EEPROM(Epaule); Ecrire_un_Entier_en_EEPROM(Hanche);
      Calcule_Nb_Enregistrements();}} // Calcule et affiche l'écran1.

void Liste_les_enregistrements_utilises() {
  boolean Trouve = false;
  PTR = 474; Compteur_caracteres = 1; Serial.println();
  while (PTR < 1023) {
    if (Lire_un_Octet_en_EEPROM()) {Serial.print(Compteur_caracteres); Serial.println(" valide."); Trouve = true;}
    PTR = PTR + 10; Compteur_caracteres++;}
    if (!Trouve) {BIP(); Serial.println("Aucun enregistrement.");}}
    
void Kill_enregistrement() {// Ne fait que forcer à "00 le premier octet du groupe.
  Verifie_1_a_50(); Positionne_PTR_en_fonction_de_Num(); // Pointe le début de l'enregistrement.
  if (num_ERR == 0) {Ecrire_un_Octet_en_EEPROM(0); Calcule_Nb_Enregistrements();}}
  
//-----------------------------------------------------------------------------------
void CADRES() {u8g.drawFrame(0,0,128,13); u8g.drawFrame(0,12,128,52);} // Cadre TITRE et cadre données.
  
void Aff_TEXTE(byte X, byte Y, char TEXTE[]) {// Choisir Y=9 pour tous les titres.
  char Caractere; byte I = 0; Caractere = 'x'; // 'x' pour ne pas avoir 0 en entrée de boucle.
  while (Caractere !=0) {Caractere = TEXTE[I]; u8g.setPrintPos((I*6)+X, Y); I++; u8g.print(Caractere);}}
  
//-----------------------------------------------------------------------------------

void Lister_EEPROM() {
  PTR = 0;
  Lister_un_BLOC();Lister_un_BLOC();
  Lister_un_BLOC();Lister_un_BLOC(); }
  
void Lister_un_BLOC() {
  Lister_ENTETE();
  byte Compteur_lignes = 0;
  while (Compteur_lignes < 16)
    {Lister_une_LIGNE(); Compteur_lignes++; } }
  
void Lister_une_LIGNE() {   
  byte Compteur_Octets = 0; Serial.print(' ');
  if (PTR<10) {Serial.print('0');};
  if (PTR<100) {Serial.print('0');};
  if (PTR<1000) {Serial.print('0');};
  Serial.print(PTR); Serial.print("  ");  
  while (Compteur_Octets < 16) {
    Lister_OCTET(); Compteur_Octets++; }
  Serial.println(); }
  
void Lister_ENTETE() {
   Serial.println();
   Serial.println(" ADRS  00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F");}
   
byte Lire_Octet_EEPROM(int ADRESSE) {return eeprom_read_byte((unsigned char*) ADRESSE);}

void Ecrire_Octet_EEPROM(int ADRESSE, byte Octet) {eeprom_write_byte((unsigned char*) ADRESSE, Octet);}
   
void Lister_OCTET() {
  byte octet = Lire_Octet_EEPROM(PTR);
  if (octet < 16) Serial.print('0'); 
  Serial.print(octet,HEX);
  Serial.print(' '); PTR++;}






